Package: DelayedRandomArray
Version: 1.20.0
Date: 2024-07-05
Title: Delayed Arrays of Random Values
Description: 
    Implements a DelayedArray of random values where the realization of the sampled values is delayed until they are needed.
    Reproducible sampling within any subarray is achieved by chunking where each chunk is initialized with a different random seed and stream.
    The usual distributions in the stats package are supported, along with scalar, vector and arrays for the parameters.
Authors@R: person("Aaron", "Lun", role=c("aut", "cre"), email="infinite.monkeys.with.keyboards@gmail.com")
License: GPL-3
Depends: SparseArray (>= 1.5.15), DelayedArray (>= 0.31.6)
Imports: methods, dqrng, Rcpp
LinkingTo: dqrng, BH, Rcpp
Suggests: testthat, knitr, BiocStyle, rmarkdown, Matrix
biocViews: DataRepresentation
BugReports: https://github.com/LTLA/DelayedRandomArray/issues
URL: https://github.com/LTLA/DelayedRandomArray
SystemRequirements: C++11
VignetteBuilder: knitr
RoxygenNote: 7.3.2
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/DelayedRandomArray
git_branch: RELEASE_3_23
git_last_commit: e4e339e
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 21:40:23 UTC; biocbuild
Author: Aaron Lun [aut, cre]
Maintainer: Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>
